DTree

Enum DTree 

pub enum DTree {
    Node {
        l: Box<DTree>,
        r: Box<DTree>,
        cutset: VarSet,
        vars: VarSet,
    },
    Leaf {
        clause: Vec<Literal>,
        cutset: VarSet,
        vars: VarSet,
    },
}

Variants



Node

Fields

l: Box<DTree>

left child

r: Box<DTree>

right child

cutset: VarSet

cutset: vars(l) ∩ vars(r) \ ancestor cutsets in English: set of variables that are in both left and right children that are not cut by an ancestor

vars: VarSet

vars = vars(l) ∪ vars(r)



Leaf

Fields

clause: Vec<Literal>

each leaf associated with a clause

cutset: VarSet
vars: VarSet

vars is simply set of vars in the clause

Implementations

Source

impl DTree

pub fn from_cnf(cnf: &Cnf, elim_order: &VarOrder) -> DTree

given an elimination order elim_order, generate a corresponding dtree. follows algorithm 25, eo2dtree, from Darwiche

pub fn cutwidth(&self) -> usize

computes the cutwidth of the dtree, which is the size of the largest cut

the width is a measure of the complexity of elimination for this dtree